/****************************************************************************
 Title	:   EA-C20017AR LCD library
 Author:    Hubert Gschwandtner
 File:	    V 1.0
 Software:  AVR-GCC 3.3 
 Target:    

 DESCRIPTION
       Basic routines for interfacing a Epson SED1200F-based text lcd display

	   Der Code ist abgeleitet von der HD44780U LCD library von Peter Fleury

       Auf die Abfrage des Busy-Flag wurde verzichtet, da sonst 1 zustzliche Pin
	   vom Kontroller bentigt wird. CS wird nur bentigt wenn mehrere Display parallel 
	   angeschlossen werden, liegt daher dauernd auf GND. RD liegt dauernd auf VCC.

 USAGE
       See the C include lcd_epson.h file for a description of each function
 LICENCE 
  		Copyright (C) 2008  Hubert Gschwandtner

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.    
*****************************************************************************/
#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include "lcd_epson.h"



/* 
** constants/macros 
*/
#define DDR(x) (*(&x - 1))      /* address of data direction register of port x */
#if defined(__AVR_ATmega64__) || defined(__AVR_ATmega128__)
    /* on ATmega64/128 PINF is on port 0x00 and not 0x60 */
    #define PIN(x) ( &PORTF==&(x) ? _SFR_IO8(0x00) : (*(&x - 2)) )
#else
	#define PIN(x) (*(&x - 2))    /* address of input register of port x          */
#endif



#define lcd_wr_delay()   __asm__ __volatile__( "rjmp 1f\n 1:" );
#define lcd_wr_high()    LCD_WR_PORT  |=  _BV(LCD_WR_PIN);
#define lcd_wr_low()     LCD_WR_PORT  &= ~_BV(LCD_WR_PIN);
#define lcd_wr_toggle()  toggle_wr()

#define lcd_a0_high()   LCD_A0_PORT |=  _BV(LCD_A0_PIN)
#define lcd_a0_low()    LCD_A0_PORT &= ~_BV(LCD_A0_PIN)



/* 
** function prototypes 
*/
#if LCD_IO_MODE
static void toggle_wr(void);
#endif
volatile uint8_t pos;

/*
** local functions
*/



/*************************************************************************
 delay loop for small accurate delays: 16-bit counter, 4 cycles/loop
*************************************************************************/
static inline void _delayFourCycles(unsigned int __count)
{
    if ( __count == 0 )    
        __asm__ __volatile__( "rjmp 1f\n 1:" );    // 2 cycles
    else
        __asm__ __volatile__ (
    	    "1: sbiw %0,1" "\n\t"                  
    	    "brne 1b"                              // 4 cycles/loop
    	    : "=w" (__count)
    	    : "0" (__count)
    	   );
}


/************************************************************************* 
delay for a minimum of <us> microseconds
the number of loops is calculated at compile-time from MCU clock frequency
*************************************************************************/
#define delay(us)  _delayFourCycles( ( ( 1*(F_CPU/4000) )*us)/1000 )


/* toggle WR Pin to initiate write */
static void toggle_wr(void)
{
  	delay(5);
	lcd_wr_low();
   	lcd_wr_delay();
	lcd_wr_high();
    
	
}



/*************************************************************************
Low-level function to write byte to LCD controller
Input:    data   byte to write to LCD
          a0     1: write data    
                 0: write instruction
Returns:  none
*************************************************************************/

static void lcd_write(uint8_t data,uint8_t a0)
{

        /* configure data pins as output */
        DDR(LCD_DATA0_PORT) |= _BV(LCD_DATA0_PIN);
        DDR(LCD_DATA1_PORT) |= _BV(LCD_DATA1_PIN);
        DDR(LCD_DATA2_PORT) |= _BV(LCD_DATA2_PIN);
        DDR(LCD_DATA3_PORT) |= _BV(LCD_DATA3_PIN);
        
        /* output high nibble first */

        LCD_DATA3_PORT &= ~_BV(LCD_DATA3_PIN);
        LCD_DATA2_PORT &= ~_BV(LCD_DATA2_PIN);
        LCD_DATA1_PORT &= ~_BV(LCD_DATA1_PIN);
        LCD_DATA0_PORT &= ~_BV(LCD_DATA0_PIN);
    	if(data & 0x80) LCD_DATA3_PORT |= _BV(LCD_DATA3_PIN);
    	if(data & 0x40) LCD_DATA2_PORT |= _BV(LCD_DATA2_PIN);
    	if(data & 0x20) LCD_DATA1_PORT |= _BV(LCD_DATA1_PIN);
    	if(data & 0x10) LCD_DATA0_PORT |= _BV(LCD_DATA0_PIN);

	if (a0==0){
		lcd_a0_high();
		}
	else{
		lcd_a0_low();
		}
		
        lcd_wr_toggle();	
        
        /* output low nibble */

        LCD_DATA3_PORT &= ~_BV(LCD_DATA3_PIN);
        LCD_DATA2_PORT &= ~_BV(LCD_DATA2_PIN);
        LCD_DATA1_PORT &= ~_BV(LCD_DATA1_PIN);
        LCD_DATA0_PORT &= ~_BV(LCD_DATA0_PIN);
    	if(data & 0x08) LCD_DATA3_PORT |= _BV(LCD_DATA3_PIN);
    	if(data & 0x04) LCD_DATA2_PORT |= _BV(LCD_DATA2_PIN);
    	if(data & 0x02) LCD_DATA1_PORT |= _BV(LCD_DATA1_PIN);
    	if(data & 0x01) LCD_DATA0_PORT |= _BV(LCD_DATA0_PIN);

	if (a0==0){
		lcd_a0_low();
		}
	else {
		lcd_a0_high();

		}		
        lcd_wr_toggle();        
    
        /* all data pins high (inactive) */
        LCD_DATA0_PORT |= _BV(LCD_DATA0_PIN);
        LCD_DATA1_PORT |= _BV(LCD_DATA1_PIN);
        LCD_DATA2_PORT |= _BV(LCD_DATA2_PIN);
        LCD_DATA3_PORT |= _BV(LCD_DATA3_PIN);

		lcd_a0_low();

	if(a0 ==1){
		pos++;
		
		if(pos == 0x8a){
			lcd_command(0xc0);
			}
		else if(pos ==0xca){
			lcd_command(0xb0);
			}
		}
}
/*************************************************************************
loops while lcd is busy, returns address counter
*************************************************************************/

static uint8_t lcd_waitbusy(void)

{
	  	delay(lcd_busy_time);
		
		return 0;
}


/* lcd_waitbusy */



/*************************************************************************
Send LCD controller instruction command
Input:   instruction to send to LCD controller, see HD44780 data sheet
Returns: none
*************************************************************************/
void lcd_command(uint8_t cmd)
{
    lcd_waitbusy();
    lcd_write(cmd,0);
}


/*************************************************************************
Send data byte to LCD controller 
Input:   data to send to LCD controller, see HD44780 data sheet
Returns: none
*************************************************************************/
void lcd_data(uint8_t data)
{
    lcd_waitbusy();
    lcd_write(data,1);
}



/*************************************************************************
Set cursor to specified position
Input:    x  horizontal position  (0: left most position)
          y  vertical position    (0: first line)
Returns:  none
*************************************************************************/
void lcd_gotoxy(uint8_t x, uint8_t y)
{

if (x >=10){
	x=x-10;
	x=x | 0xc0;
	pos=x;
	}
else {
	x=x | 0x80;
	pos=x;
	}
	lcd_command (x);

}/* lcd_gotoxy */


/*************************************************************************
*************************************************************************/
int lcd_getxy(void)
{
    return lcd_waitbusy();
}


/*************************************************************************
Clear display and set cursor to home position
*************************************************************************/
void lcd_clrscr(void){
	uint8_t xy;

    lcd_command(LCD_CURSOR_HOME);
	pos=0x80;			/* cursor home */
	for(xy=0x80; xy<=0x8a; xy++){
	lcd_putc(0x20);
	}
	lcd_command(0xc0);
	for(xy=0xc0; xy<=0xca; xy++){
	lcd_putc(0x20);
	}
	lcd_command(LCD_CURSOR_HOME);			/* cursor home */
	pos=0x80;
}


/*************************************************************************
Set cursor to home position
*************************************************************************/
void lcd_home(void)
{
    lcd_command(LCD_CURSOR_HOME);
}


/*************************************************************************
Display character at current cursor position 
Input:    character to be displayed                                       
Returns:  none
*************************************************************************/
void lcd_putc( char c)
{

        lcd_write(c, 1);

}/* lcd_putc */


/*************************************************************************
Display string without auto linefeed 
Input:    string to be displayed
Returns:  none
*************************************************************************/
void lcd_puts(const char *s)
/* print string on lcd (no auto linefeed) */
{
    register  char c;

    while ( (c = *s++) ) {
        lcd_putc(c);
    }

}/* lcd_puts */


/*************************************************************************
Display string from program memory without auto linefeed 
Input:     string from program memory be be displayed                                        
Returns:   none
*************************************************************************/
void lcd_puts_p(const char *progmem_s)
/* print string from program memory on lcd (no auto linefeed) */
{
    register char c;

    while ( (c = pgm_read_byte(progmem_s++)) ) {
        lcd_putc(c);
    }

}/* lcd_puts_p */


/*************************************************************************
Initialize display and select type of cursor 
Input:    dispAttr LCD_DISP_OFF            display off
                   LCD_DISP_ON             display on, cursor off
                   LCD_DISP_ON_CURSOR      display on, cursor on
                   LCD_DISP_CURSOR_BLINK   display on, cursor on flashing
Returns:  none
*************************************************************************/
void lcd_init(uint8_t dispAttr)
{

 delay(lcd_power_on_time);        /* wait 16ms or more after power-on       */
 delay(lcd_power_on_time);        /* wait 16ms or more after power-on       */
 delay(lcd_power_on_time);        /* wait 16ms or more after power-on       */
 delay(lcd_power_on_time);        /* wait 16ms or more after power-on       
 				Das Display braucht sehr lang bis es aktiv ist	*/    
     
    	LCD_WR_PORT  |=  _BV(LCD_WR_PIN);
        DDR(LCD_WR_PORT)     |= _BV(LCD_WR_PIN);
		DDR(LCD_A0_PORT)     |= _BV(LCD_A0_PIN);
        DDR(LCD_DATA0_PORT) |= _BV(LCD_DATA0_PIN);
        DDR(LCD_DATA1_PORT) |= _BV(LCD_DATA1_PIN);
        DDR(LCD_DATA2_PORT) |= _BV(LCD_DATA2_PIN);
        DDR(LCD_DATA3_PORT) |= _BV(LCD_DATA3_PIN);
		
   
 delay(lcd_power_on_time);        /* wait 16ms or more after power-on       */
	
    
   
    

	
    lcd_command(LCD_SYS_RESET);      		/* System Reset  				 */
	lcd_command(LCD_LINE_SELECT); 			/* Zeilenauswahl				 */
    lcd_command(LCD_DISP_ON);        	    /* display on                    */
    lcd_command(LCD_DISP_ON_CURSOR_ON);     /* display on, cursor on         */
    lcd_command(LCD_DISP_ON_CURSOR_BLINK_ON);/* display/cursor control       */
	lcd_clrscr();
	pos=0x80;
}

/* lcd_init */
