
/*************************************************************************
 Title	:   EA-C20017AR LCD library
 Author:    Hubert Gschwandtner
 File:	    V 1.0
 Software:  AVR-GCC 3.3 
 Hardware:  any AVR device, 
***************************************************************************/

/**
 DESCRIPTION
 		Basic routines for interfacing a Epson SED1200F-based text lcd display

	   	Der Code ist abgeleitet von der HD44780U LCD library von Peter Fleury
 
 LICENCE
        
Copyright (C) 2008  Hubert Gschwandtner

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*@{*/

#if (__GNUC__ * 100 + __GNUC_MINOR__) < 303
#error "This library requires AVR-GCC 3.3 or later, update to newer AVR-GCC compiler !"
#endif

#include <inttypes.h>
#include <avr/pgmspace.h>

/** 
 *  @name  Definitions for MCU Clock Frequency
 *  Adapt the MCU clock frequency in Hz to your target. 
 */
#ifndef F_CPU
#define F_CPU 16000000UL  /* clock frequency in Hz, used to calculate delay timer */
#endif


#define lcd_busy_time	5000	/* busy_time 5000sec */
#define lcd_power_on_time  16000  /* Startverzgerung bei Power On !max 16000! */


 
#define LCD_PORT         PORTB        /**< port for the LCD lines   */
#define LCD_DATA0_PORT   LCD_PORT     /**< port for 4bit data bit 0 */
#define LCD_DATA1_PORT   LCD_PORT     /**< port for 4bit data bit 1 */
#define LCD_DATA2_PORT   LCD_PORT     /**< port for 4bit data bit 2 */
#define LCD_DATA3_PORT   LCD_PORT     /**< port for 4bit data bit 3 */
#define LCD_DATA0_PIN    0            /**< pin for 4bit data bit 0  */
#define LCD_DATA1_PIN    1            /**< pin for 4bit data bit 1  */
#define LCD_DATA2_PIN    2            /**< pin for 4bit data bit 2  */
#define LCD_DATA3_PIN    3            /**< pin for 4bit data bit 3  */
#define LCD_A0_PORT      LCD_PORT     /**< port for A0 line         */
#define LCD_A0_PIN       4            /**< pin  for A0 line         */
#define LCD_WR_PORT      LCD_PORT     /**< port for Enable line     */
#define LCD_WR_PIN       5            /**< pin  for Enable line     */




/* display on/off, cursor on/off, blinking char at cursor position  */
#define LCD_SYS_RESET				0x10	/* Sys Reset	*/
#define LCD_LINE_SELECT				0x13	/* single line	*/
#define LCD_DISP_OFF             		0x0C   /* display off   */
#define LCD_DISP_ON              		0x0D   /* display on, cursor off*/
#define LCD_DISP_ON_CURSOR_OFF  		0x0E   /* display on, cursor off*/
#define LCD_DISP_ON_CURSOR_ON    		0x0F   /* display on, cursor on */
#define LCD_DISP_ON_CURSOR_BLINK_OFF 0x0A   /* disp on, cursor on, blink off*/  
#define LCD_DISP_ON_CURSOR_BLINK_ON	 0x0B   /* disp on, cursor on, blink on */
#define LCD_CURSOR_HOME					0x80	/* cursor 00	*/

/* move cursor/shift display */
#define LCD_MOVE_CURSOR_LEFT     0x05   /* move cursor left  (decrement)    */
#define LCD_MOVE_CURSOR_RIGHT    0x04   /* move cursor right (increment)    */
#define LCD_MOVE_DISP_LEFT       0x07   /* shift display left               */
#define LCD_MOVE_DISP_RIGHT      0x06   /* shift display right              */


/** 
 *  @name Functions
 */


/**
 @brief    Initialize display and select type of cursor
 @param    dispAttr \b LCD_DISP_OFF display off\n
                    \b LCD_DISP_ON display on, cursor off\n
                    \b LCD_DISP_ON_CURSOR display on, cursor on\n
                    \b LCD_DISP_ON_CURSOR_BLINK display on, cursor on flashing             
 @return  none
*/
extern void lcd_init(uint8_t dispAttr);


/**
 @brief    Clear display and set cursor to home position
 @param    void                                        
 @return   none
*/
extern void lcd_clrscr(void);


/**
 @brief    Set cursor to home position
 @param    void                                        
 @return   none
*/
extern void lcd_home(void);


/**
 @brief    Set cursor to specified position
 
 @param    x horizontal position\n (0: left most position)
 @param    y vertical position\n   must be always 0
 @return   none
*/
extern void lcd_gotoxy(uint8_t x, uint8_t y);


/**
 @brief    Display character at current cursor position
 @param    c character to be displayed                                       
 @return   none
*/
extern void lcd_putc( char c);


/**
 @brief    Display string without auto linefeed
 @param    s string to be displayed                                        
 @return   none
*/
extern void lcd_puts(const  char *s);


/**
 @brief    Display string from program memory without auto linefeed
 @param    s string from program memory be be displayed                                        
 @return   none
 @see      lcd_puts_P
*/
extern void lcd_puts_p(const  char *progmem_s);


/**
 @brief    Send LCD controller instruction command
 @param    cmd instruction to send to LCD controller, 
 @return   none
*/
extern void lcd_command(uint8_t cmd);


/**
 @brief    Send data byte to LCD controller 
 
 Similar to lcd_putc(), but without interpreting LF
 @param    data byte to send to LCD controller, 
 @return   none
*/
extern void lcd_data(uint8_t data);


/**
 @brief macros for automatically storing string constant in program memory
*/
#define lcd_puts_P(__s)         lcd_puts_p(PSTR(__s))

/*@}*/

